@@ -25,7 +25,7 @@ except ImportError:
     logger.warn('decorator module is not installed')
 
 
-from datetime import timedelta
+from datetime import timedelta, datetime
 
 try:
     import numpy as np
@@ -48,7 +48,7 @@ except ImportError:
     DataFrame = object  # load as object, we'll fail at runtime
     logger.warn('pandas module is not installed')
 
-from metrique.utils import dt2ts, utcnow, ts2dt
+from metrique.utils import dt2ts, ts2dt
 
 
 def filtered(f):
@@ -75,12 +75,12 @@ def filtered(f):
 
 class Result(DataFrame):
     ''' Custom DataFrame implementation for Metrique '''
-    def __init__(self, data=None, date=None):
+    def __init__(self, data=None, date=None, **kwargs):
         if not HAS_PANDAS:
             raise RuntimeError("`pip install pandas` required")
         if not HAS_NUMPY:
             raise RuntimeError("`pip install numpy` required")
-        super(Result, self).__init__(data)
+        super(Result, self).__init__(data, **kwargs)
         if data is not None and len(data) and \
                 any([c not in self for c in ['_oid', '_start', '_end']]):
             raise ValueError('_oid, _start and _end must be defined')
@@ -101,6 +101,24 @@ class Result(DataFrame):
         if '_end' in self.columns:
             self._end_isnull = self._end.isnull()
 
+    def __getitem__(self, key):
+        try:
+            return super(Result, self).__getitem__(key)
+        except KeyError as e:
+            if not len(self):
+                return pd.Series(dtype=object)
+            else:
+                raise e
+
+    def __getattr__(self, key):
+        try:
+            return super(Result, self).__getattr__(key)
+        except AttributeError as e:
+            if not len(self):
+                return pd.Series(dtype=object)
+            else:
+                raise e
+
     def to_datetime(self, column):
         '''
         This function converts epoch timestamps to datetimes.
@@ -227,7 +245,7 @@ class Result(DataFrame):
         end = Timestamp(end or max(Timestamp(self._end.max()),
                                    self._start.max()))
         # FIXME: end != end ?
-        end = utcnow() if repr(end) == 'NaT' else end
+        end = datetime.utcnow() if repr(end) == 'NaT' else end
         start = start if self.check_in_bounds(start) else self._lbound
         end = end if self.check_in_bounds(end) else self._rbound
 
@@ -360,7 +378,7 @@ class Result(DataFrame):
                 max_start_ser_map[oid] = row
 
         vals = max_start_ser_map.values()
-        cut_ts = utcnow()
+        cut_ts = datetime.utcnow()
         ages = []
         for row in vals:
             end = row[i_end]
