@@ -29,6 +29,7 @@ from collections import defaultdict, Mapping, OrderedDict
 from copy import copy
 import cPickle
 import cProfile as profiler
+import csv
 from datetime import datetime, date
 try:
     from dateutil.parser import parse as dt_parse
@@ -103,6 +104,7 @@ INVALID_USERNAME_RE = re.compile('[^a-zA-Z_]')
 HOME_DIR = os.environ.get('METRIQUE_HOME')
 PREFIX_DIR = os.environ.get('METRIQUE_PREFIX')
 LOGS_DIR = os.environ.get('METRIQUE_LOGS')
+ETC_DIR = os.environ.get('METRIQUE_ETC')
 CACHE_DIR = os.environ.get('METRIQUE_CACHE')
 SRC_DIR = os.environ.get('METRIQUE_SRC')
 BACKUP_DIR = env.get('METRIQUE_BACKUP')
@@ -236,10 +238,12 @@ def configure(options=None, defaults=None, config_file=None,
     sk = section_key
 
     if (section_key or section_only) and not (sk and sk in config):
-        raise KeyError('section %s not set' % sk)
-
-    # work only with the given section, if specified
-    working_config = config[sk] if sk in config else config
+        #raise KeyError('section %s not set' % sk)
+        logger.warn('section %s not set' % sk)
+        working_config = {}
+    else:
+        # work only with the given section, if specified
+        working_config = config[sk] if sk in config else config
 
     # if section key is already configured, ie, we initiated with
     # config set already, set options not set as None
@@ -339,6 +343,20 @@ def cube_pkg_mod_cls(cube):
     return pkg, mod, _cls
 
 
+def _data_export(data, as_df=False):
+    is_true(HAS_PANDAS, "`pip install pandas` required")
+    if as_df:
+        if isinstance(data, pd.DataFrame):
+            return data
+        else:
+            return pd.DataFrame(data)
+    else:
+        if isinstance(data, pd.DataFrame):
+            return data.T.to_dict().values()
+        else:
+            return data
+
+
 def _debug_set_level(logger, level):
     # NOTE: int(0) == bool(False) is True
     if level in [-1, 0, False]:
@@ -502,7 +520,7 @@ def get_cube(cube, init=False, pkgs=None, cube_paths=None, config=None,
 
 
 def get_pid(pid_file=None):
-    if not pid_file:
+    if not (pid_file and os.path.exists(pid_file)):
         return 0
     return int(''.join(open(pid_file).readlines()).strip())
 
@@ -609,18 +627,18 @@ def is_defined(value, msg=None, except_=None):
     return is_true(result, msg=msg, except_=except_)
 
 
-def is_empty(value, msg=None, except_=None):
+def is_empty(value, msg=None, except_=None, inc_zeros=True):
     '''
     is defined, but null or empty like value
     '''
-    # 0, 0.0, 0L are also considered 'empty'
     if hasattr(value, 'empty'):
         # dataframes must check for .empty
         # since they don't define truth value attr
         # take the negative, since below we're
         # checking for cases where value 'is_null'
         value = not bool(value.empty)
-    elif value in ZEROS:
+    elif inc_zeros and value in ZEROS:
+        # also consider 0, 0.0, 0L as 'empty'
         # will check for the negative below
         value = True
     else:
@@ -721,81 +739,6 @@ def list2str(items, delim=','):
     return item
 
 
-def load_file(path, filetype=None, as_df=False, **kwargs):
-    if not os.path.exists(path):
-        raise IOError("%s does not exist" % path)
-    if not filetype:
-        # try to get file extension
-        filetype = path.split('.')[-1]
-    if filetype in ['csv', 'txt']:
-        result = load_csv(path, **kwargs)
-    elif filetype in ['json']:
-        result = load_json(path, **kwargs)
-    elif filetype in ['pickle']:
-        result = load_pickle(path, **kwargs)
-    else:
-        raise TypeError("Invalid filetype: %s" % filetype)
-    return _data_export(result, as_df=as_df)
-
-
-def load_pickle(path, **kwargs):
-    result = []
-    with open(path) as f:
-        while 1:
-            # in case we have multiple pickles dumped
-            try:
-                result.append(cPickle.load(f))
-            except EOFError:
-                break
-    return result
-
-
-def load_csv(path, **kwargs):
-    is_true(HAS_PANDAS, "`pip install pandas` required")
-    kwargs.setdefault('skipinitialspace', True)
-    # load the file according to filetype
-    return pd.read_csv(path, **kwargs)
-
-
-def load_json(path, **kwargs):
-    is_true(HAS_PANDAS, "`pip install pandas` required")
-    return pd.read_json(path, **kwargs)
-
-
-def local_tz():
-    if time.daylight:
-        offsetHour = time.altzone / 3600
-    else:
-        offsetHour = time.timezone / 3600
-    return 'Etc/GMT%+d' % offsetHour
-
-
-def local_tz_to_utc(dt):
-    dt = ts2dt(dt)
-    convert = get_timezone_converter(_local_tz)
-    return convert(dt)
-
-
-def set_oid_func(_oid_func):
-    k = itertools.count(1)
-
-    def __oid_func(o):
-        ''' default __oid generator '''
-        o['_oid'] = o['_oid'] if '_oid' in o else k.next()
-        return o
-
-    if _oid_func:
-        if _oid_func is True:
-            _oid_func = __oid_func
-        elif isfunction(_oid_func):
-            pass
-        else:
-            raise TypeError("_oid must be a function!")
-    else:
-        _oid_func = None
-    return _oid_func
-
-
 def load(path, filetype=None, as_df=False, retries=None,
          _oid=None, quiet=False, **kwargs):
     '''Load multiple files from various file types automatically.
@@ -836,6 +779,8 @@ def load(path, filetype=None, as_df=False, retries=None,
     else:
         path = re.sub('^file://', '', path)
         path = os.path.expanduser(path)
+        # assume relative to cwd if not already absolute path
+        path = path if os.path.isabs(path) else pjoin(os.getcwd(), path)
         files = sorted(glob.glob(os.path.expanduser(path)))
         if not files:
             raise IOError("failed to load: %s" % path)
@@ -860,18 +805,20 @@ def load(path, filetype=None, as_df=False, retries=None,
         return objects
 
 
-def _data_export(data, as_df=False):
-    is_true(HAS_PANDAS, "`pip install pandas` required")
-    if as_df:
-        if isinstance(data, pd.DataFrame):
-            return data
+def load_config(path):
+    if not path:
+        return {}
+
+    config_file = os.path.expanduser(path)
+    if HAS_ANYCONFIG:
+        conf = anyconfig.load(config_file) or {}
+        if conf:
+            # convert mergeabledict (anyconfig) to dict of dicts
+            return conf.convert_to(conf)
         else:
-            return pd.DataFrame(data)
+            raise IOError("Invalid config file: %s" % config_file)
     else:
-        if isinstance(data, pd.DataFrame):
-            return data.T.to_dict().values()
-        else:
-            return data
+        raise RuntimeError("`pip install anyconfig` required!")
 
 
 def _load_cube_pkg(pkg, cube):
@@ -895,23 +842,72 @@ def _load_cube_pkg(pkg, cube):
         return getattr(mcubes, _cls)
 
 
-def load_config(path):
-    if not path:
-        return {}
-
-    config_file = os.path.expanduser(path)
-    if HAS_ANYCONFIG:
-        conf = anyconfig.load(config_file) or {}
-        if conf:
-            # convert mergeabledict (anyconfig) to dict of dicts
-            return conf.convert_to(conf)
-        else:
-            raise IOError("Invalid config file: %s" % config_file)
+def load_csv(path, use_pandas=False, **kwargs):
+    if use_pandas:
+        is_true(HAS_PANDAS, "`pip install pandas` required")
+        kwargs.setdefault('skipinitialspace', True)
+        # load the file according to filetype
+        return pd.read_csv(path, **kwargs)
     else:
-        raise RuntimeError("`pip install anyconfig` required!")
+        _csv = open(path, 'rU')
+        sample = _csv.read(1024)
+        has_header = csv.Sniffer().has_header(sample) if sample else False
+        _csv.seek(0)
+        is_true(has_header, "csv doesn't have a header defining field names!")
+        fields = csv.reader(_csv).next()
+        return csv.DictReader(_csv, fieldnames=fields, **kwargs)
+
+
+def load_file(path, filetype=None, as_df=False, **kwargs):
+    if not os.path.exists(path):
+        raise IOError("%s does not exist" % path)
+    if not filetype:
+        # try to get file extension
+        filetype = path.split('.')[-1]
+    if filetype in ['csv', 'txt']:
+        result = load_csv(path, **kwargs)
+    elif filetype in ['json']:
+        result = load_json(path, **kwargs)
+    elif filetype in ['pickle']:
+        result = load_pickle(path, **kwargs)
+    else:
+        raise TypeError("Invalid filetype: %s" % filetype)
+    return _data_export(result, as_df=as_df)
+
+
+def load_json(path, **kwargs):
+    is_true(HAS_PANDAS, "`pip install pandas` required")
+    return pd.read_json(path, **kwargs)
+
+
+def load_pickle(path, **kwargs):
+    result = []
+    with open(path) as f:
+        while 1:
+            # in case we have multiple pickles dumped
+            try:
+                result.append(cPickle.load(f))
+            except EOFError:
+                break
+    return result
+
+
+def local_tz():
+    if time.daylight:
+        offsetHour = time.altzone / 3600
+    else:
+        offsetHour = time.timezone / 3600
+    return 'Etc/GMT%+d' % offsetHour
+
+
+def local_tz_to_utc(dt):
+    dt = ts2dt(dt)
+    convert = get_timezone_converter(_local_tz)
+    return convert(dt)
 
 
 def make_dirs(path, mode=0700, quiet=True):
+    path = path or ''
     if not path.startswith('/'):
         raise OSError("requires absolute path! got %s" % path)
     if os.path.exists(path):
@@ -971,7 +967,21 @@ def rand_chars(size=6, chars=string.ascii_uppercase + string.digits,
     return chars
 
 
-def read_file(rel_path, paths=None, raw=False, as_list=False, *args, **kwargs):
+def read_in_chunks(file_object, chunk_size=1024):
+    """
+    Lazy function (generator) to read a file piece by piece.
+    Default chunk size: 1k.
+    See: http://stackoverflow.com/a/519653
+    """
+    while True:
+        data = file_object.read(chunk_size)
+        if not data:
+            break
+        yield data
+
+
+def read_file(rel_path, paths=None, raw=False, as_list=False, as_iter=False,
+              *args, **kwargs):
     '''
         find a file that lives somewhere within a set of paths and
         return its contents. Default paths include 'static_dir'
@@ -991,11 +1001,16 @@ def read_file(rel_path, paths=None, raw=False, as_list=False, *args, **kwargs):
             break
     else:
         raise IOError("path %s does not exist!" % rel_path)
+    args = args if args else ['rU']
     fd = open(path, *args, **kwargs)
     if raw:
         return fd
 
-    fd_lines = fd.readlines()
+    if as_iter:
+        return read_in_chunks(fd)
+    else:
+        fd_lines = fd.readlines()
+
     if as_list:
         return fd_lines
     else:
@@ -1075,10 +1090,36 @@ def safestr(str_):
     return "".join(x for x in str_ if x.isalnum())
 
 
+def set_oid_func(_oid_func):
+    k = itertools.count(1)
+
+    def __oid_func(o):
+        ''' default __oid generator '''
+        o['_oid'] = o['_oid'] if '_oid' in o else k.next()
+        return o
+
+    if _oid_func:
+        if _oid_func is True:
+            _oid_func = __oid_func
+        elif isfunction(_oid_func):
+            pass
+        else:
+            raise TypeError("_oid must be a function!")
+    else:
+        _oid_func = None
+    return _oid_func
+
+
 def sha1_hexdigest(o):
     return sha1(repr(o)).hexdigest()
 
 
+def ssl_gen(cert_path, key_path):
+    logger.debug("Generating self-signed SSL certificate + key + combined pem")
+    sys_call('openssl req -new -x509 -days 365 -nodes '
+             '-out %s -keyout %s -batch' % (cert_path, key_path))
+
+
 def str2list(item, delim=',', map_=None):
     if isinstance(item, basestring):
         items = item.split(delim)
@@ -1128,6 +1169,8 @@ def sys_call(cmd, sig=None, sig_func=None, shell=True, cwd=None, quiet=False,
              fork=False, pid_file=None, ignore_errors=False, bg=False):
     _path = os.getcwd()
     cwd = cwd or _path
+    is_true(bool(not fork or (fork and pid_file)),
+            'pid_file must be defined when forking!')
     try:
         if fork:
             logger.warn('*' * 50 + 'FORKING' + '*' * 50)
@@ -1353,11 +1396,22 @@ def virtualenv_activate(virtenv=None):
         raise OSError("Invalid virtual env; %s not found" % activate_this)
 
 
-def write_file(path, value, mode='w', force=False):
-    if os.path.exists(path) and mode == 'w' and not force:
-        raise RuntimeError('file exists, use different mode or force=True')
+def write_file(path, value, mode='w', force=False, exists_ext='new'):
+    is_true(mode in ('w', 'a'), 'invalid write mode: %s' % mode)
+    exists = os.path.exists(path)
+    is_w = mode == 'w'
+    # force if we set it to true or if exists_ext is None
+    if exists and is_w and not force:
+        if exists_ext:
+            # doesn't overwrite the existing file, but write the new
+            # file with an extension
+            path = '%s.%s' % (path, exists_ext)
+            logger.warn("File exists, saving instead as %s" % path)
+        else:
+            raise RuntimeError("File exists; use force=True to overwrite")
     with open(path, mode) as f:
         f.write(unicode(str(value)))
+    logger.info("Installed %s ..." % path)
 
 
 class DictDiffer(object):
